"""
A package that contains the data types that are used in the library.
\n
Copyright © 2023 Kalynovsky Valentin. All rights reserved.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

from .animation_type import IndefiniteAnimationType, DefiniteAnimationType
from .entry_type import EntryType
from .environment_type import EnvironmentType
from .sorting_key_type import SortingKeyType
from .text_buffer_type import TextBufferType
